'''
This script empowers the protocol to quantify the carbon footprint in research projects

Assumptions (can be modified as function argument):
- Project duration: 14 weeks
- Duration of a virtual meeting: 45 min
- Power consumption of the main device used during the research
(screening time, video-conferencing): 40 W

User-provided information (as function argument):
- Weekly screening time in s (influences carbon footprint of screening time and meetings)
- Carbon footprint of due to data traffic through your Internet browser (from Carbonalyser) in gCO2
- Carbon cost of ML simulations (using CUMULATOR base tool) in gCO2

Source: project report of Tristan Trébaol made in Spring 2020.
'''

# Heuristics

# total energy consumption during virtual meeting
virtual_meeting_consumption = 187.6  # in W
# carbon intensity (avg in EU in 2014)
carbon_intensity = 447  # in gCO2eq/kWh
# 1byte model in Wh/byte
one_byte_model = 6.894E-11
# Average WLTP emissions in (km/gCO2)
car = 1/148.1

# Method

# prints and returns the total carbon footprint
def project_carbon_footprint(carbonalyser, ml_simulations, screening_time,
                             n_weeks=14, meeting_duration=45, hardware_consumption=40):
    meetups = joule_to_kwh(virtual_meeting_consumption * \
                           meeting_duration * 60 * \
                           n_weeks) * carbon_intensity
    screening = joule_to_kwh(n_weeks * screening_time * hardware_consumption) * carbon_intensity
    # total carbon footprint in gC02eq
    carbon_emissions = meetups + screening + carbonalyser + ml_simulations
    car_distance = car * carbon_emissions

    print('##############################################################'
          '\nTotal carbon emissions for this project: %s gCO2eq'
          '\nYou could have powered the average passenger car for %i km'
          '\nwith all that compute effort.'
          '\n--------------------------------------------------------------' % ("{:.2e}".format(carbon_emissions),
                                                                                car_distance))
    print('ML models: %s gCO2eq' % "{:.2e}".format(ml_simulations))
    print('Internet data traffic: %s gCO2eq' % "{:.2e}".format(carbonalyser))
    print('Meetings: %s gCO2eq' % "{:.2e}".format(meetups))
    print('Screening: %s gCO2eq' % "{:.2e}".format(screening))
    print('--------------------------------------------------------------'
          '\n##############################################################')
    return carbon_emissions


# energy conversion
def joule_to_kwh(value):
    return value / 3.6e6



'''
This is the base class of cumulator.
'''
import time as t

# assumptions to approximate the carbon footprint
# computation costs: consumption of a typical GPU in Watts converted to kWh/s
hardware_load = 250/3.6e6
# communication costs: average energy impact of traffic in a typical data centers, kWh/byte
one_byte_model = 6.894E-11
# conversion to carbon footprint: average carbon intensity value in gCO2eq/kWh in the EU in 2014
carbon_intensity = 447


class Cumulator:

    def __init__(self, n_processors=1):
        self.t0 = 0
        self.t1 = 0
        # times are in seconds
        self.time_list = []
        self.cumulated_time = 0
        # file sizes are in bytes
        self.file_size_list = []
        self.cumulated_data_traffic = 0
	self.n_processors = n_processors

    # starts accumulating time
    def on(self):
        self.t0 = t.time()

    # stops accumulating time and records the value
    def off(self):
        self.t1 = t.time()
        self.cumulated_time += self.t1 - self.t0
        self.time_list.append(self.t1 - self.t0)

    # records the amount of data transferred, file_size in bytes
    def data_transferred(self, file_size):
        self.file_size_list.append(file_size)
        self.cumulated_data_traffic += file_size

    # computes time based carbon footprint due to computations
    def computation_costs(self):
        return self.cumulated_time * self.n_processors * hardware_load * carbon_intensity

    # computes the carbon footprint due to communication
    def communication_costs(self):
        return one_byte_model * self.cumulated_data_traffic

    # computes the total carbon footprint
    def total_carbon_footprint(self):
        return self.computation_costs() + self.communication_costs()

    # prints the carbon footprint in the terminal
    def display_carbon_footprint(self):
        print('########\nOverall carbon footprint: %s gCO2eq\n########' %
              "{:.2e}".format(self.total_carbon_footprint()))
        print('Carbon footprint due to computations: %s gCO2eq' %
              "{:.2e}".format(self.computation_costs()))
        print('Carbon footprint due to communications: %s gCO2eq' %
              "{:.2e}".format(self.communication_costs()))